package groowt.view.component.compiler;

import groowt.view.component.ViewComponent;

import java.util.HashMap;
import java.util.Map;

public abstract class CachingComponentTemplateCompiler<U extends ComponentTemplateCompileUnit>
        implements ComponentTemplateCompiler<U> {

    private final Map<Class<? extends ViewComponent>, ComponentTemplateCompileResult> cache = new HashMap<>();

    @Override
    public final ComponentTemplateCompileResult compile(U compileUnit) throws ComponentTemplateCompileException {
        if (this.cache.containsKey(compileUnit.getForClass())) {
            return this.cache.get(compileUnit.getForClass());
        } else {
            final ComponentTemplateCompileResult compileResult = this.doCompile(compileUnit);
            this.cache.put(compileUnit.getForClass(), compileResult);
            return compileResult;
        }
    }

    protected abstract ComponentTemplateCompileResult doCompile(U compileUnit) throws ComponentTemplateCompileException;

}
